#include <PalmOS.h>				
#include "yoda.h"

UInt32 NextBlinkTime;
Boolean Blinking = false;

/*********************************************************************/
/*********************************************************************/
static Err StartApplication(void)
{
  FrmGotoForm(YodaForm);
  NextBlinkTime = TimGetTicks();
  SysRandom(NextBlinkTime);
  return 0;
}

/*********************************************************************/
/*********************************************************************/
static void StopApplication(void)
{
  FrmCloseAllForms();
}

/*********************************************************************/
/*********************************************************************/
void DrawBitmap(Int16 x, Int16 y, Int16 id)
{
  MemHandle h;
  BitmapPtr p;

  h = DmGet1Resource('Tbmp', id);
  if (h != NULL) {
    p = MemHandleLock(h);
    WinDrawBitmap(p, x, y);
    MemHandleUnlock(h);
  }
  DmReleaseResource(h);
}

/*********************************************************************/
/*********************************************************************/
static Int16 RandInt(Int16 MaxNo)
{
  // returns 0 <= int <= MaxNo-1
  return(SysRandom(0) % MaxNo);
}

/*********************************************************************/
/*********************************************************************/
static void SetFieldTextFromStr(UInt16 fieldID, Int8 * strP)
{
  MemHandle txtH, oldTxtH;
  FormPtr frm = FrmGetActiveForm();
  FieldPtr fldP;

  txtH = MemHandleNew(StrLen(strP) + 1);
  if (txtH) {
    StrCopy(MemHandleLock(txtH), strP);
    MemHandleUnlock(txtH);
    fldP = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, fieldID));
    oldTxtH = FldGetTextHandle(fldP);
    FldSetTextHandle(fldP, txtH);
    FldDrawField(fldP);
    if (oldTxtH)
      MemHandleFree(oldTxtH);
  }
}

/*********************************************************************/
/*********************************************************************/
static Boolean YodaFormHandleEvent(EventPtr event)
{
  static Int16 oldrnd = 0;
  Boolean handled = false;
  Int16 rnd;
  MemHandle h;
  Char *s;

  switch (event->eType) {
  case ctlSelectEvent:  // A control button was pressed and released.
    if (event->data.ctlEnter.controlID==WisdomBtn) {
      rnd = 3000+RandInt(34);
      while(rnd==oldrnd) rnd = 3000+RandInt(34);
      oldrnd = rnd;
      h = DmGetResource('tSTR', rnd);
      s = MemHandleLock(h);
      SetFieldTextFromStr(YodaField, s);
      MemHandleUnlock(h);
      DmReleaseResource(h);
      handled = true;
    }
    if (event->data.ctlEnter.controlID==QuestionBtn) {
      rnd = 4000+RandInt(11);
      while(rnd==oldrnd) rnd = 4000+RandInt(11);
      oldrnd=rnd;
      h = DmGetResource('tSTR', rnd);
      s = MemHandleLock(h);
      SetFieldTextFromStr(YodaField, s);
      MemHandleUnlock(h);
      DmReleaseResource(h);
      handled = true;
    }
    break;

  case nilEvent:
    if (Blinking) DrawBitmap(30, 56, 101);
    if (TimGetTicks() >= NextBlinkTime) {
      if (RandInt(2)) {
	DrawBitmap(30, 56, 102+RandInt(2));
	Blinking = true;
      }
      NextBlinkTime = NextBlinkTime + (2*SysTicksPerSecond() - 25);
      handled = true;
    }
    break;

  case frmOpenEvent:	
    FrmDrawForm(FrmGetActiveForm());
    break;
	
  case menuEvent:
    if (event->data.menu.itemID==5002) {
      FrmHelp(5000);
      handled = true;
    }
    break;
  }			
  return(handled);
}

/*********************************************************************/
/*********************************************************************/
static Boolean ApplicationHandleEvent(EventPtr event)
{
  FormPtr frm;
  Int16	formId;
  Boolean handled = false;

  if (event->eType == frmLoadEvent) {
    // Load the form resource specified in the event then activate the form.
    formId = event->data.frmLoad.formID;
    frm = FrmInitForm(formId);
    FrmSetActiveForm(frm);

    // Set the event handler for the form.  The handler of the currently 
    // active form is called by FrmDispatchEvent each time it receives an event
    switch (formId) {
    case YodaForm:
      FrmSetEventHandler(frm, YodaFormHandleEvent);
      handled = true;
      break;
    }
  }
  return handled;
}

/*********************************************************************/
/*********************************************************************/
static void EventLoop(void)
{
  EventType event;
  UInt16 error;
	
  do {
    EvtGetEvent(&event, SysTicksPerSecond() / 2);
    if (!SysHandleEvent(&event))
      if (!MenuHandleEvent(0, &event, &error))
	if (!ApplicationHandleEvent(&event))
	  FrmDispatchEvent(&event);
  }
  while (event.eType != appStopEvent);
}

/*********************************************************************/
/*********************************************************************/
UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
  Err error = 0;		
  if (cmd == sysAppLaunchCmdNormalLaunch) {
    if ((error = StartApplication()) == 0) {   
      EventLoop();
      StopApplication();
    }
  }
  return error;
}
